--[[	PhantomPlates NamePlate Text Module
	by SDPhantom
	https://www.wowinterface.com/forums/member.php?u=34145
	https://www.curseforge.com/members/sdphantomgamer/projects	]]
--------------------------------------------------------------------------

--------------------------
--[[	Namespace	]]
--------------------------
local AddOn=select(2,...);

----------------------------------
--[[	Import From Utilities	]]
----------------------------------
local AddOn_Utilities_UnitPowerColor=AddOn.Utilities.UnitPowerColor;

----------------------------------
--[[	Local References	]]
----------------------------------
local math_abs=math.abs;
local math_floor=math.floor;
local math_log10=math.log10;
local math_max=math.max;

----------------------------------
--[[	Number Formatting	]]
----------------------------------
local NumberCaps={FIRST_NUMBER_CAP,SECOND_NUMBER_CAP};
local AddOn_Localization_NumberGroupingScale=AddOn.Localization.NumberGroupingScale;
local function AbbreviateNumber(val)--	Abbreviates large numbers
	local exp=math_max(0,math_floor(math_log10(math_abs(val))));--	Calculate exponent of 10 and clamp to zero
	if exp<AddOn_Localization_NumberGroupingScale then return tostring(math_floor(val)); end--	Less than 1k, return as-is

	local factor=math_floor(exp/AddOn_Localization_NumberGroupingScale);--	Exponent factor of 1k
	local precision=math_max(0,(AddOn_Localization_NumberGroupingScale-1)-exp%AddOn_Localization_NumberGroupingScale);--	Dynamic precision based on how many digits we have (Returns numbers like 100k, 10.0k, and 1.00k)

--	Fallback to scientific notation if we run out of units
	return (("%s%%0.%df%%s"):format(val<0 and "-" or "",precision)):format(val/(10^AddOn_Localization_NumberGroupingScale)^factor,NumberCaps[factor] or "e"..(factor*AddOn_Localization_NumberGroupingScale));
end

------------------
--[[	Hooks	]]
------------------
local TextObjects={};

AddOn.HookNamePlateCreated(function(_,base)
	local unitframe=base.UnitFrame;
	if not TextObjects[unitframe] then
		local health=unitframe.healthBar:CreateFontString(nil,"OVERLAY");--"NumberFontNormalSmall");
		health:SetFont("Fonts\\ArialN.ttf",10,"THICKOUTLINE");--	Fonts are easier to read when made from scratch rather than resizing an inherited one
		health:SetPoint("LEFT",0,0);
		health:SetTextColor(0,1,0);

		local power=unitframe.healthBar:CreateFontString(nil,"OVERLAY");--"NumberFontNormalSmall");
		power:SetFont("Fonts\\ArialN.ttf",10,"THICKOUTLINE");--	Fonts are easier to read when made from scratch rather than resizing an inherited one
		power:SetPoint("RIGHT",0,0);
		power:SetTextColor(ColorMixin.GetRGB(PowerBarColor.MANA));

		if not AddOn.Options.NamePlateText_ShowHealth then health:Hide(); end
		if not AddOn.Options.NamePlateText_ShowPower then power:Hide(); end
		TextObjects[unitframe]={health,power};
	end
end);

--	These functions are shared with other UnitFrames
--	:OnAdded() calls CompactUnitFrame_UpdateAll(), which calls these anyway
hooksecurefunc("CompactUnitFrame_UpdateHealth",function(self)
	if not (TextObjects[self] and AddOn.Options.NamePlateText_ShowHealth) then return; end

	local health=UnitHealth(self.displayedUnit);
	if AddOn.Options.NamePlateText_AbbreviateNumber then health=AbbreviateNumber(health); end
	TextObjects[self][1]:SetText(health);
end);

hooksecurefunc("CompactUnitFrame_UpdatePower",function(self)
	if not (TextObjects[self] and AddOn.Options.NamePlateText_ShowPower) then return; end

	local power=UnitPower(self.displayedUnit,(UnitPowerType(self.displayedUnit)));
	if AddOn.Options.NamePlateText_AbbreviateNumber then power=AbbreviateNumber(power); end
	TextObjects[self][2]:SetText(power);
end);

hooksecurefunc("CompactUnitFrame_UpdatePowerColor",function(self)
	if not (TextObjects[self] and AddOn.Options.NamePlateText_ShowPower) then return; end
	TextObjects[self][2]:SetTextColor(AddOn_Utilities_UnitPowerColor(self.displayedUnit));
end);

--------------------------
--[[	Options Setup	]]
--------------------------
local OptionsButton_AbbreviateNumbers;

local function OnOptionsUpdate(_,key,val)
	local enabled=(AddOn.Options.NamePlateText_ShowHealth or AddOn.Options.NamePlateText_ShowPower);
	if key=="NamePlateText_ShowHealth" or key=="NamePlateText_ShowPower" then
		OptionsButton_AbbreviateNumbers:SetEnabled(enabled);
		OptionsButton_AbbreviateNumbers.text:SetFontObject(enabled and GameFontNormalSmall or GameFontDisableSmall);
	end

	for unitframe,text in pairs(TextObjects) do
		if key=="NamePlateText_ShowHealth" then text[1]:SetShown(val); end
		if key=="NamePlateText_ShowPower" then text[2]:SetShown(val); end
		if enabled and unitframe.displayedUnit then
			CompactUnitFrame_UpdateHealth(unitframe);
			CompactUnitFrame_UpdatePower(unitframe);
			CompactUnitFrame_UpdatePowerColor(unitframe);
		end
	end
end

AddOn.RegisterOptionsControl("NamePlateText_ShowHealth",true
	,"OnClick","SetChecked","GetChecked"
	,OnOptionsUpdate
	,function(panel)--	Initialization
		local button=CreateFrame("CheckButton",nil,panel,"UICheckButtonTemplate");
		button:SetPoint("TOPLEFT",24,-48);
		button.text:SetText(AddOn.Localization.OptionsSetting_NamePlateText_ShowHealth);
		return button;
	end
);

AddOn.RegisterOptionsControl("NamePlateText_ShowPower",true
	,"OnClick","SetChecked","GetChecked"
	,OnOptionsUpdate
	,function(panel)--	Initialization
		local button=CreateFrame("CheckButton",nil,panel,"UICheckButtonTemplate");
		button:SetPoint("TOPLEFT",24,-72);
		button.text:SetText(AddOn.Localization.OptionsSetting_NamePlateText_ShowPower);
		return button;
	end
);

AddOn.RegisterOptionsControl("NamePlateText_AbbreviateNumber",true
	,"OnClick","SetChecked","GetChecked"
	,OnOptionsUpdate
	,function(panel)--	Initialization
		local button=CreateFrame("CheckButton",nil,panel,"UICheckButtonTemplate");
		button:SetPoint("TOPLEFT",48,-96);
		button.text:SetText(AddOn.Localization.OptionsSetting_NamePlateText_AbbreviateNumber);

		OptionsButton_AbbreviateNumbers=button;
		return button;
	end
);
