--[[	PhantomPlates PowerBars Module
	by SDPhantom
	https://www.wowinterface.com/forums/member.php?u=34145
	https://www.curseforge.com/members/sdphantomgamer/projects	]]
--------------------------------------------------------------------------

--------------------------
--[[	Namespace	]]
--------------------------
local AddOn=select(2,...);

----------------------------------
--[[	Import From Utilities	]]
----------------------------------
local AddOn_Utilities_UnitPowerColor=AddOn.Utilities.UnitPowerColor;

------------------------------------------
--[[	HealthBar Color Adjustments	]]
------------------------------------------
DefaultCompactNamePlateFriendlyFrameOptions.healthBarColorOverride=nil;
DefaultCompactNamePlateFriendlyFrameOptions.useClassColors=false;
DefaultCompactNamePlateFriendlyFrameOptions.colorHealthBySelection=false;

DefaultCompactNamePlateEnemyFrameOptions.healthBarColorOverride=nil;
DefaultCompactNamePlateEnemyFrameOptions.useClassColors=false;
DefaultCompactNamePlateEnemyFrameOptions.colorHealthBySelection=false;

------------------
--[[	Hooks	]]
------------------
local PowerBars={};

AddOn.HookNamePlateCreated(function(_,base)
	local unitframe=base.UnitFrame;
	if not PowerBars[unitframe] then
		local bar=unitframe.healthBar:CreateTexture(nil,"BORDER");
		bar:SetTexture("Interface/TargetingFrame/UI-TargetingFrame-BarFill");
		bar:SetDrawLayer("BORDER",1);
		bar:SetPoint("TOPLEFT",unitframe.healthBar,"LEFT");
		bar:SetPoint("BOTTOM");
		bar:SetWidth(unitframe.healthBar:GetWidth());

		if not AddOn.Options.PowerBars_Enabled then bar:Hide(); end
		PowerBars[unitframe]=bar;
	end
end);

--	These functions are shared with other UnitFrames
local function UpdatePower(self)
	local bar,displayedunit=PowerBars[self],self.displayedUnit;
	if not (bar and AddOn.Options.PowerBars_Enabled) then return; end

	local powertype=UnitPowerType(displayedunit);
	local cur,max=UnitPower(displayedunit,powertype),UnitPowerMax(displayedunit,powertype);
	if max>0 then
		local percent=cur/max;
		local width=percent*self.healthBar:GetWidth();
		if width>=1 then
			bar:Show();
			bar:SetWidth(width);
			bar:SetTexCoord(0,percent,0.5,1);
		else bar:Hide(); end
	else bar:Hide(); end
end

local function UpdatePowerColor(self)
	if not (PowerBars[self] and AddOn.Options.PowerBars_Enabled) then return; end
	PowerBars[self]:SetVertexColor(AddOn_Utilities_UnitPowerColor(self.displayedUnit));
end

--	:OnAdded() calls CompactUnitFrame_UpdateAll(), which calls these anyway
hooksecurefunc("CompactUnitFrame_UpdateMaxPower",UpdatePower);
hooksecurefunc("CompactUnitFrame_UpdatePower",UpdatePower);
hooksecurefunc("CompactUnitFrame_UpdatePowerColor",UpdatePowerColor);

--------------------------
--[[	Options Setup	]]
--------------------------
AddOn.RegisterOptionsControl("PowerBars_Enabled",true
	,"OnClick","SetChecked","GetChecked"
	,function(_,_,val)--	Callback
		for unitframe,bar in pairs(PowerBars) do
			bar:SetShown(val);
			if unitframe.displayedUnit then
				UpdatePower(unitframe);
				UpdatePowerColor(unitframe);
			end
		end
	end
	,function(panel)--	Initialization
		local button=CreateFrame("CheckButton",nil,panel,"UICheckButtonTemplate");
		button:SetPoint("TOPLEFT",24,-144);
		button.text:SetText(AddOn.Localization.OptionsSetting_PowerBars_Enabled);
		return button;
	end
);
